@@ -35,7 +35,7 @@ def get_enabled():
 
         salt '*' service.get_enabled
     '''
-    prefix = '/etc/rc{0}.d/S'.format(_get_runlevel())
+    prefix = '/etc/rc[S{0}].d/S'.format(_get_runlevel())
     ret = set()
     lines = glob.glob('{0}*'.format(prefix))
     for line in lines:
@@ -156,6 +156,9 @@ def enable(name, **kwargs):
         salt '*' service.enable <service name>
     '''
     cmd = 'update-rc.d {0} enable'.format(name)
+    osmajor = __grains__['osrelease'].split('.')[0]
+    if int(osmajor) >= 6:
+        cmd = 'insserv {0} && '.format(name) + cmd
     return not __salt__['cmd.retcode'](cmd)
 
 
