@@ -10,7 +10,7 @@ import os
 import salt.utils
 
 
-grainmap = {
+GRAINMAP = {
            'Arch': '/etc/rc.d',
            'Debian': '/etc/init.d',
            'Fedora': '/etc/init.d',
@@ -21,6 +21,8 @@ grainmap = {
            'CloudLinux': '/etc/init.d',
            'Amazon': '/etc/init.d',
            'SunOS': '/etc/init.d',
+           'SUSE  Enterprise Server': '/etc/init.d',
+           'OEL': '/etc/init.d',
           }
 
 def __virtual__():
@@ -39,6 +41,8 @@ def __virtual__():
                'Ubuntu',
                'Debian',
                'Arch',
+               'ALT',
+               'OEL',
               ]
     if __grains__['os'] in disable:
         return False
@@ -56,7 +60,7 @@ def start(name):
 
         salt '*' service.start <service name>
     '''
-    cmd = os.path.join(grainmap[__grains__['os']],
+    cmd = os.path.join(GRAINMAP[__grains__['os']],
             name + ' start')
     return not __salt__['cmd.retcode'](cmd)
 
@@ -69,7 +73,7 @@ def stop(name):
 
         salt '*' service.stop <service name>
     '''
-    cmd = os.path.join(grainmap[__grains__['os']],
+    cmd = os.path.join(GRAINMAP[__grains__['os']],
             name + ' stop')
     return not __salt__['cmd.retcode'](cmd)
 
@@ -84,7 +88,7 @@ def restart(name):
     '''
     if name == 'salt-minion':
         salt.utils.daemonize_if(__opts__)
-    cmd = os.path.join(grainmap[__grains__['os']],
+    cmd = os.path.join(GRAINMAP[__grains__['os']],
             name + ' restart')
     return not __salt__['cmd.retcode'](cmd)
 
@@ -110,6 +114,6 @@ def reload(name):
 
         salt '*' service.reload <service name>
     '''
-    cmd = os.path.join(grainmap[__grains__['os']],
+    cmd = os.path.join(GRAINMAP[__grains__['os']],
             name + ' reload')
     return not __salt__['cmd.retcode'](cmd)
