@@ -27,6 +27,19 @@ __outputter__ = {
 log = logging.getLogger(__name__)
 
 
+def __resolve_struct(value, kwval_as):
+    '''
+    Take a string representing a structure and safely serialize it with the
+    specified medium
+    '''
+    if kwval_as == 'yaml':
+        return  _yaml_load(value, _YamlCustomLoader)
+    elif kwval_as == 'json':
+        return json.loads(value)
+    elif kwval_as is None or kwval_as == 'verbatim':
+        return value
+
+
 def running():
     '''
     Return a dict of state return data if a state function is already running.
@@ -132,8 +145,16 @@ def highstate(test=None, **kwargs):
     if not test is None:
         opts['test'] = test
 
-    st_ = salt.state.HighState(opts)
-    ret = st_.call_highstate()
+    pillar = __resolve_struct(
+            kwargs.get('pillar', ''),
+            kwargs.get('kwval_as', 'yaml'))
+
+    st_ = salt.state.HighState(opts, pillar)
+    st_.push_active()
+    try:
+        ret = st_.call_highstate(exclude=kwargs.get('exclude', []))
+    finally:
+        st_.pop_active()
     serial = salt.payload.Serial(__opts__)
     cache_file = os.path.join(__opts__['cachedir'], 'highstate.p')
 
@@ -149,7 +170,7 @@ def highstate(test=None, **kwargs):
     return ret
 
 
-def sls(mods, env='base', test=None, **kwargs):
+def sls(mods, env='base', test=None, exclude=None, **kwargs):
     '''
     Execute a set list of state modules from an environment, default
     environment is base
@@ -166,18 +187,33 @@ def sls(mods, env='base', test=None, **kwargs):
     if not test is None:
         opts['test'] = test
 
+    pillar = __resolve_struct(
+            kwargs.get('pillar', ''),
+            kwargs.get('kwval_as', 'yaml'))
+
     salt.utils.daemonize_if(opts, **kwargs)
-    st_ = salt.state.HighState(opts)
+    st_ = salt.state.HighState(opts, pillar)
 
     if isinstance(mods, string_types):
         mods = mods.split(',')
 
-    high, errors = st_.render_highstate({env: mods})
+    st_.push_active()
+    try:
+        high, errors = st_.render_highstate({env: mods})
 
-    if errors:
-        return errors
+        if errors:
+            return errors
 
-    ret = st_.state.call_high(high)
+        if exclude:
+            if isinstance(exclude, str):
+                exclude = exclude.split(',')
+            if '__exclude__' in high:
+                high['__exclude__'].extend(exclude)
+            else:
+                high['__exclude__'] = exclude
+        ret = st_.state.call_high(high)
+    finally:
+        st_.pop_active()
     serial = salt.payload.Serial(__opts__)
     cache_file = os.path.join(__opts__['cachedir'], 'sls.p')
     try:
@@ -201,8 +237,12 @@ def top(topfn):
     if conflict:
         return conflict
     st_ = salt.state.HighState(__opts__)
+    st_.push_active()
     st_.opts['state_top'] = os.path.join('salt://', topfn)
-    return st_.call_highstate()
+    try:
+        return st_.call_highstate()
+    finally:
+        st_.pop_active()
 
 
 def show_highstate():
@@ -255,6 +295,10 @@ def show_sls(mods, env='base', test=None, **kwargs):
 def show_top():
     '''
     Return the top data that the minion will use for a highstate
+
+    CLI Example::
+
+        salt '*' state.show_top
     '''
     st_ = salt.state.HighState(__opts__)
     return st_.get_top()
@@ -311,6 +355,8 @@ def single(fun, name, test=None, kwval_as='yaml', **kwargs):
     elif kwval_as == 'json':
         def parse_kwval(value):
             return json.loads(value)
+    elif kwval_as is None or kwval_as == 'verbatim':
+        parse_kwval = lambda value: value
     else:
         return 'Unknown format({0}) for state keyword arguments!'.format(
                 kwval_as)
