@@ -33,6 +33,7 @@ def managed(name, **kwargs):
         The name of the package repo, as it would be referred to when running
         the regular package manager commands.
 
+
     For yum-based systems, take note of the following configuration values:
 
     humanname
@@ -53,6 +54,58 @@ def managed(name, **kwargs):
         Sometimes you want to supply additional information, but not as
         enabled configuration. Anything supplied for this list will be saved
         in the repo configuration with a comment marker (#) in front.
+
+    Additional configuration values, such as gpgkey or gpgcheck, are used
+    verbatim to update the options for the yum repo in question.
+
+
+    For apt-based systems, take note of the following configuration values:
+
+    name:
+        on apt-based systems this must be the complete entry as it would be
+        seen in the sources.list file.  This can have a limited subset of
+        components (i.e. 'main') which can be added/modified with the
+        "comps" option.
+
+          EXAMPLE: deb http://us.archive.ubuntu.com/ubuntu/ precise main
+
+    disabled
+        On apt-based systems, disabled toggles whether or not the repo is
+        used for resolving dependancies and/or installing packages
+
+    comps
+        On apt-based systems, comps dictate the types of packages to be
+        installed from the repository (e.g. main, nonfree, ...).  For
+        purposes of this, comps should be a comma-separated list.
+
+    file
+       The filename for the .list that the repository is configured in.
+       It is important to include the full-path AND make sure it is in
+       a directory that APT will look in when handling packages
+
+    dist
+       This dictates the release of the distro the packages should be built
+       for.  (e.g. unstable)
+
+    keyid
+       The KeyID of the GPG key to install.  This option also requires
+       the 'keyserver' option to be set.
+
+    keyserver
+       This is the name of the keyserver to retrieve gpg keys from.  The
+       keyid option must also be set for this option to work.
+
+    key_url
+       A web url to retreive the GPG key from.
+
+    consolidate:
+       If set to true, this will consolidate all sources definitions to
+       the sources.list file, cleanup the now unused files, consolidate
+       components (e.g. main) for the same uri, type, and architecture
+       to a single line, and finally remove comments from the sources.list
+       file.  The consolidate will run every time the state is processed. The
+       option only needs to be set on one repo managed by salt to take effect.
+
     '''
     ret = {'name': name,
            'changes': {},
@@ -63,20 +116,6 @@ def managed(name, **kwargs):
         repo = __salt__['pkg.get_repo'](name)
     except:
         pass
-    if repo:
-        notset = False
-        for kwarg in kwargs:
-            if kwarg not in repo.keys():
-                notset = True
-            else:
-                if kwargs[kwarg] != repo[kwarg]:
-                    notset = True
-        if notset is False:
-            ret['comment'] = 'Package repo {0} already configured'.format(name)
-            return ret
-    if __opts__['test']:
-        ret['comment'] = 'Package repo {0} needs to be configured'.format(name)
-        return ret
 
     # pkg.mod_repo has conflicting kwargs, so move 'em around
     repokwargs = {}
@@ -85,22 +124,50 @@ def managed(name, **kwargs):
             repokwargs['repo'] = kwargs[kwarg]
         elif kwarg == 'humanname':
             repokwargs['name'] = kwargs[kwarg]
-        elif kwarg in ('__id__', 'fun', 'state'):
+        elif kwarg in ('__id__', 'fun', 'state', '__env__', '__sls__', 'order'):
             pass
         else:
             repokwargs[kwarg] = kwargs[kwarg]
 
-    __salt__['pkg.mod_repo'](repo=name, **repokwargs)
+    if repo:
+        notset = False
+        for kwarg in repokwargs:
+            if kwarg not in repo.keys():
+                notset = True
+            else:
+                if repokwargs[kwarg] != repo[kwarg]:
+                    notset = True
+        if notset is False:
+            ret['comment'] = 'Package repo {0} already configured'.format(name)
+            return ret
+    if __opts__['test']:
+        ret['comment'] = 'Package repo {0} needs to be configured'.format(name)
+        return ret
+    try:
+        __salt__['pkg.mod_repo'](repo=name, **repokwargs)
+    except Exception, e:
+        # This is another way to pass information back from the mod_repo
+        # function.
+        ret['result'] = False
+        ret['comment'] = 'Failed to configure repo "{0}": {1}'.format(name,
+                                                                      str(e))
+        return ret
     try:
         repodict = __salt__['pkg.get_repo'](name)
-        ret['changes'] = {'repo': name}
+        if repo:
+            for kwarg in repokwargs:
+                if repodict.get(kwarg) != repo.get(kwarg):
+                    change = { 'new': repodict[kwarg],
+                               'old': repo.get(kwarg) }
+                    ret['changes'][kwarg] = change
+        else:
+            ret['changes'] = { 'repo': name }
         ret['result'] = True
         ret['comment'] = 'Configured package repo {0}'.format(name)
-        return ret
-    except:
-        pass
-    ret['result'] = False
-    ret['comment'] = 'Failed to configure repo {0}'.format(name)
+    except Exception, e:
+        ret['result'] = False
+        ret['comment'] = 'Failed to confirm config of repo {0}: {1}'.format(
+            name, str(e))
     return ret
 
 
