@@ -61,6 +61,28 @@ class PipModuleTest(integration.ModuleCase):
         self.assertEqual(ret['retcode'], 0)
         self.assertIn('uninstalled pep8', ret['stdout'])
 
+    def test_pip_install_upgrade(self):
+        # Create the testing virtualenv
+        self.run_function('virtualenv.create', [self.venv_dir])
+        ret = self.run_function(
+            'pip.install', ['pep8==1.3.4'], bin_env=self.venv_dir
+        )
+        self.assertEqual(ret['retcode'], 0)
+        self.assertIn('installed pep8', ret['stdout'])
+        ret = self.run_function(
+            'pip.install',
+            ['pep8'],
+            bin_env=self.venv_dir,
+            upgrade=True
+        )
+        self.assertEqual(ret['retcode'], 0)
+        self.assertIn('installed pep8', ret['stdout'])
+        ret = self.run_function(
+            'pip.uninstall', ['pep8'], bin_env=self.venv_dir
+        )
+        self.assertEqual(ret['retcode'], 0)
+        self.assertIn('uninstalled pep8', ret['stdout'])
+
     def tearDown(self):
         super(PipModuleTest, self).tearDown()
         if os.path.isdir(self.venv_test_dir):
