@@ -14,7 +14,7 @@ import tempfile
 
 # Import salt libs
 import saltunittest
-from integration import print_header, PNUM, TestDaemon
+from integration import print_header, PNUM, TestDaemon, TMP
 
 try:
     import xmlrunner
@@ -22,7 +22,10 @@ except ImportError:
     xmlrunner = None
 
 TEST_DIR = os.path.dirname(os.path.normpath(os.path.abspath(__file__)))
-
+XML_OUTPUT_DIR = os.environ.get(
+    'SALT_XML_TEST_REPORTS_DIR',
+    os.path.join(TMP, 'xml-test-reports')
+)
 
 try:
     import coverage
@@ -56,12 +59,15 @@ def run_suite(opts, path, display_name, suffix='[!_]*.py'):
     print_header('Starting {0}'.format(header))
 
     if opts.xmlout:
-        runner = xmlrunner.XMLTestRunner(output='test-reports').run(tests)
+        runner = xmlrunner.XMLTestRunner(output=XML_OUTPUT_DIR).run(tests)
     else:
+        if not os.path.isdir(XML_OUTPUT_DIR):
+            os.makedirs(XML_OUTPUT_DIR)
         runner = saltunittest.TextTestRunner(
             verbosity=opts.verbosity
         ).run(tests)
         TEST_RESULTS.append((header, runner))
+
     return runner.wasSuccessful()
 
 
@@ -138,107 +144,154 @@ def parse_opts():
     Parse command line options for running specific tests
     '''
     parser = optparse.OptionParser()
-    parser.add_option('-m',
-            '--module',
-            '--module-tests',
-            dest='module',
-            default=False,
-            action='store_true',
-            help='Run tests for modules')
-    parser.add_option('-S',
-            '--state',
-            '--state-tests',
-            dest='state',
-            default=False,
-            action='store_true',
-            help='Run tests for states')
-    parser.add_option('-c',
-            '--client',
-            '--client-tests',
-            dest='client',
-            default=False,
-            action='store_true',
-            help='Run tests for client')
-    parser.add_option('-s',
-            '--shell',
-            dest='shell',
-            default=False,
-            action='store_true',
-            help='Run shell tests')
-    parser.add_option('-r',
-            '--runner',
-            dest='runner',
-            default=False,
-            action='store_true',
-            help='Run runner tests')
-    parser.add_option('-u',
-            '--unit',
-            '--unit-tests',
-            dest='unit',
-            default=False,
-            action='store_true',
-            help='Run unit tests')
-    parser.add_option('-v',
-            '--verbose',
-            dest='verbosity',
-            default=1,
-            action='count',
-            help='Verbose test runner output')
-    parser.add_option('-x',
-            '--xml',
-            dest='xmlout',
-            default=False,
-            action='store_true',
-            help='XML test runner output')
-    parser.add_option('-n',
-            '--name',
-            dest='name',
-            action='append',
-            default=[],
-            help='Specific test name to run')
-    parser.add_option('--clean',
-            dest='clean',
-            default=True,
-            action='store_true',
-            help=('Clean up test environment before and after '
-                  'integration testing (default behaviour)'))
-    parser.add_option('--no-clean',
-            dest='clean',
-            action='store_false',
-            help=('Don\'t clean up test environment before and after '
-                  'integration testing (speed up test process)'))
-    parser.add_option('--run-destructive',
-            action='store_true',
-            default=False,
-            help='Run destructive tests. These tests can include adding or '
-                 'removing users from your system for example. Default: '
-                 '%default'
+
+    parser.add_option(
+        '--sysinfo',
+        default=False,
+        action='store_true',
+        help='Print some system information.'
     )
-    parser.add_option('--no-report',
-            default=False,
-            action='store_true',
-            help='Do NOT show the overall tests result'
+
+    tests_select_group = optparse.OptionGroup(
+        parser,
+        "Tests Selection Options",
+        "Select which tests are to be executed"
     )
-    parser.add_option('--coverage',
-            default=False,
-            action='store_true',
-            help='Run tests and report code coverage'
+    tests_select_group.add_option(
+        '-m',
+        '--module',
+        '--module-tests',
+        dest='module',
+        default=False,
+        action='store_true',
+        help='Run tests for modules'
     )
-    parser.add_option('--sysinfo',
-            default=False,
-            action='store_true',
-            help='Print some system information.'
+    tests_select_group.add_option(
+        '-S',
+        '--state',
+        '--state-tests',
+        dest='state',
+        default=False,
+        action='store_true',
+        help='Run tests for states'
     )
-    parser.add_option('--no-colors',
-            default=False,
-            action='store_true',
-            help='Disable colour printing.')
+    tests_select_group.add_option(
+        '-c',
+        '--client',
+        '--client-tests',
+        dest='client',
+        default=False,
+        action='store_true',
+        help='Run tests for client'
+    )
+    tests_select_group.add_option(
+        '-s',
+        '--shell',
+        dest='shell',
+        default=False,
+        action='store_true',
+        help='Run shell tests'
+    )
+    tests_select_group.add_option(
+        '-r',
+        '--runner',
+        dest='runner',
+        default=False,
+        action='store_true',
+        help='Run runner tests'
+    )
+    tests_select_group.add_option(
+        '-u',
+        '--unit',
+        '--unit-tests',
+        dest='unit',
+        default=False,
+        action='store_true',
+        help='Run unit tests'
+    )
+    tests_select_group.add_option(
+        '-n',
+        '--name',
+        dest='name',
+        action='append',
+        default=[],
+        help='Specific test name to run'
+    )
+    tests_select_group.add_option(
+        '--run-destructive',
+        action='store_true',
+        default=False,
+        help='Run destructive tests. These tests can include adding or '
+             'removing users from your system for example. Default: %default'
+    )
+    parser.add_option_group(tests_select_group)
+
+    fs_cleanup_options_group = optparse.OptionGroup(
+        parser, "File system cleanup Options"
+    )
+    fs_cleanup_options_group.add_option(
+        '--clean',
+        dest='clean',
+        default=True,
+        action='store_true',
+        help='Clean up test environment before and after integration '
+             'testing (default behaviour)'
+    )
+    fs_cleanup_options_group.add_option(
+        '--no-clean',
+        dest='clean',
+        action='store_false',
+        help='Don\'t clean up test environment before and after integration '
+             'testing (speed up test process)'
+    )
+
+    output_options_group = optparse.OptionGroup(parser, "Output Options")
+    output_options_group.add_option(
+        '-v',
+        '--verbose',
+        dest='verbosity',
+        default=1,
+        action='count',
+        help='Verbose test runner output'
+    )
+    output_options_group.add_option(
+        '-x',
+        '--xml',
+        dest='xmlout',
+        default=False,
+        action='store_true',
+        help='XML test runner output(Output directory: {0})'.format(
+            XML_OUTPUT_DIR
+        )
+    )
+    output_options_group.add_option(
+        '--no-report',
+        default=False,
+        action='store_true',
+        help='Do NOT show the overall tests result'
+    )
+    output_options_group.add_option(
+        '--coverage',
+        default=False,
+        action='store_true',
+        help='Run tests and report code coverage'
+    )
+    output_options_group.add_option(
+        '--no-colors',
+        '--no-colours',
+        default=False,
+        action='store_true',
+        help='Disable colour printing.'
+    )
+    parser.add_option_group(output_options_group)
 
     options, _ = parser.parse_args()
 
     if options.xmlout and xmlrunner is None:
-        parser.error('\'--xml\' is not available. The xmlrunner library '
-                     'is not installed.')
+        parser.error(
+            '\'--xml\' is not available. The xmlrunner library is not '
+            'installed.'
+        )
 
     if options.coverage and code_coverage is None:
         parser.error(
